/**
 * 	@Author Rahul Jonna <rjonna@rjonna.com>
 *	@Version 1.0
 *	@copyright www.rjonna.com	
 */

//************************************ UTILITY FUNCTIONS & VARIABLES ************************************************

if (typeof gbmPrefNames == 'undefined') 
{
	//names of the preferences
	var gbmPrefNames =
	{
		prefUserName: "gbookmarks.username",
		prefAutoSign: "gbookmarks.autosign",
		prefRememberPwd: "gbookmarks.rememberpwd",
		prefSort: "gbookmarks.sort"
	}
}


//web response object to handle communication with the server
function WebResponse(eId, eMesg)
{
    this.eId = eId;
    this.eMesg = eMesg;
    this.results = null;
	this.auxObj = null;
	this.headers = null;
	
	function SetError(eId, eMesg)
	{
		this.eId = eId;
		this.eMesg = eMesg;
	}
} 



if (typeof gbmPrefHandler == 'undefined') 
{
	//preference handler which wraps the actual 'pref' functions 
	var gbmPrefHandler =
	{
		//used to set the preferences
		prefs : Components.classes["@mozilla.org/preferences-service;1"].
				getService(Components.interfaces.nsIPrefBranch ),
	               
		isExists : function (prefName, type)
		{
    		var prefs = this.prefs;
			type = (type == null) ? "char" : type;
			if (type == "char")
			{
				if (prefs.getPrefType(prefName) == prefs.PREF_STRING && jsDictUtils.trimWhitespace(prefs.getCharPref(prefName).toString()) != "")
					return true;
				else
					return false;
			}
			else if (type == "bool")
			{
				try
				{
					var tempValue = prefs.getBoolPref(prefName);
				}
				catch (ex)
				{
					return false;
				}
				return true;
			}
		},
		getPref : function (prefName, type)
		{
    		var prefs = this.prefs;
			try
			{
				type = (type == null) ? "char" : type;
				if (type == "char")
				{
					return prefs.getCharPref(prefName).toString();
				}
				else if (type == "bool")
				{
					return prefs.getBoolPref(prefName);
				}
			}
			catch (ex)
			{
				return "";
			}
		},
		setPref : function (prefName, value, type)
		{
    		var prefs = this.prefs;
			type = (type == null) ? "char" : type;
			if (type == "char")
			{
				prefs.setCharPref(prefName, value);
			}
			else if (type == "bool")
			{
				prefs.setBoolPref(prefName, value);
			}
		},
		setPrefIfNotExists : function (prefName, value, type)
		{
    		var prefs = this.prefs;
			type = (type == null) ? "char" : type;
			if (type == "char")
			{
				if (prefs.getPrefType(prefName) != prefs.PREF_STRING || (prefs.getPrefType(prefName) == prefs.PREF_STRING && jsDictUtils.trimWhitespace(prefs.getCharPref (prefName).toString()) == ""))
				{
					prefs.setCharPref(prefName, value);
				}

			}
			else if (type == "bool")
			{
				try
				{
					var boolValue = prefs.getBoolPref(prefName);
				}
				catch (ex)
				{
					prefs.setBoolPref(prefName, value);
				}
	            
	                
			}
		}
	}
}





 
if (typeof jsBMUtils == 'undefined') 
{
	//a utility variable 
	var jsBMUtils =
	{
		// Remove leading and trailing whitespace from a string
		trimWhitespace : function (string)
		{
			var newString  = '';
			var substring  = '';
			var beginningFound = false;

			// copy characters over to a new string
			// retain whitespace characters if they are between other characters
			for (var i = 0; i < string.length; i++)
			{
				// copy non-whitespace characters
				if (string.charAt(i) != ' ' && string.charCodeAt(i) != 9)
				{
					// if the temporary string contains some whitespace characters,copy them first
					if (substring != '')
					{
						newString += substring;
						substring = '';
					}
					newString += string.charAt(i);
					if (beginningFound == false)
						beginningFound = true;
				}
				// hold whitespace characters in a temporary string if they follow a non-whitespace character
				else if (beginningFound == true)
					substring += string.charAt(i);
			}
			return newString;
		},
	    
		getDateString :	function (dt)
		{
			dt = Math.abs(dt / 1000);
			if (dt == null || dt == undefined)
				return "";
			var modTime = new Date(dt);

			if (isNaN(modTime.getDate()))
			{
				return "";
			}
			var amORpm = (modTime.getHours() >= 0 && modTime.getHours() < 12) ? "AM" : "PM";
			var sdate = modTime.getDate();
			sdate = sdate > 9 ? sdate : "0" + sdate;
			var month = modTime.getMonth() + 1;
			month = month > 9 ? month : "0" + month;
			var hours = modTime.getHours() - 12 > 0 ? Math.abs(modTime.getHours() - 12) : modTime.getHours();    
   			hours = hours > 9 ? hours : "0" + hours;
		   	
   			var minutes = modTime.getMinutes() > 9 ? modTime.getMinutes() : "0" + modTime.getMinutes();
			return month + "/" + sdate + "/" + modTime.getFullYear()
							+ " " + hours + ":" + minutes + " " + amORpm; 

		},

		storePassword : function (userid, userpwd, remember)
		{
			var host = "chrome://gbookmarks/";
			
			var pwdManager = Components.classes["@mozilla.org/passwordmanager;1"].createInstance();

			if (pwdManager) 
			{
				pwdManager = pwdManager.QueryInterface(Components.interfaces.nsIPasswordManager);
				try
				{
					pwdManager.removeUser(host, userid);
				} 
				catch (ex) 
				{  
					
				}
				if (remember)
				{
					gBMDump(host + ", " + userid + ", " + userpwd);
					pwdManager.addUser(host, userid, userpwd);
				}
			}    
		},

		getPassword : function (userid)
		{
			var host = "chrome://gbookmarks/";
			

			var pwdManager = Components.classes["@mozilla.org/passwordmanager;1"].createInstance(Components.interfaces.nsIPasswordManagerInternal);
			var existingHost = {value:""};
			var existingUser = {value:""};
			var existingPwd = {value:""};

			try 
			{
				pwdManager.findPasswordEntry(host, userid, "", existingHost, existingUser, existingPwd);
			} 
			catch(e)
			{

			}  
			return existingPwd.value;
		}

	}
}


function gBMDump(aMessage) 
{
	var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
                                 .getService(Components.interfaces.nsIConsoleService);
	consoleService.logStringMessage("gbm: " + aMessage);
}




function gbm_nsNodeType() {
    this.target            = null;
    this.menu              = null;
    this.onTextInput       = false;
    this.onKeywordField    = false;
    this.onImage           = false;
    this.onLoadedImage     = false;
    this.onLink            = false;
    this.onMailtoLink      = false;
    this.onSaveableLink    = false;
    this.onMetaDataItem    = false;
    this.onMathML          = false;
    this.link              = false;
    this.linkURL           = "";
    this.linkURI           = null;
    this.linkProtocol      = null;
    this.inFrame           = false;
    this.hasBGImage        = false;
    this.isTextSelected    = false;
    this.isContentSelected = false;
    this.inDirList         = false;
    this.shouldDisplay     = true;
}

gbm_nsNodeType.prototype = 
{
	getNodeType : function ()
	{
		this.setTarget( document.popupNode);

        this.isTextSelected = this.isTextSelection();
        this.isContentSelected = this.isContentSelection();
	},

// Set various context menu attributes based on the state of the world.
    setTarget : function ( node ) {
        const xulNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
        if ( node.namespaceURI == xulNS ) {
          this.shouldDisplay = false;
          return;
        }

        // Initialize contextual info.
        this.onImage           = false;
        this.onLoadedImage     = false;
        this.onStandaloneImage = false;
        this.onMetaDataItem    = false;
        this.onTextInput       = false;
        this.onKeywordField    = false;
        this.imageURL          = "";
        this.onLink            = false;
        this.linkURL           = "";
        this.linkURI           = null;
        this.linkProtocol      = "";
        this.onMathML          = false;
        this.inFrame           = false;
        this.hasBGImage        = false;
        this.bgImageURL        = "";

        // Remember the node that was clicked.
        this.target = node;

        // Remember the URL of the document containing the node
        // for referrer header and for security checks.
        this.docURL = node.ownerDocument.location.href;

        // First, do checks for nodes that never have children.
        if ( this.target.nodeType == Node.ELEMENT_NODE ) {
            // See if the user clicked on an image.
            if ( this.target instanceof Components.interfaces.nsIImageLoadingContent && this.target.currentURI  ) {
                this.onImage = true;
                this.onMetaDataItem = true;
                        
                var request = this.target.getRequest( Components.interfaces.nsIImageLoadingContent.CURRENT_REQUEST );
                if (request && (request.imageStatus & request.STATUS_SIZE_AVAILABLE))
                    this.onLoadedImage = true;
                this.imageURL = this.target.currentURI.spec;

                if ( this.target.ownerDocument instanceof ImageDocument)
                   this.onStandaloneImage = true;
            } else if ( this.target instanceof HTMLInputElement ) {
               this.onTextInput = this.isTargetATextBox(this.target);
               this.onKeywordField = this.isTargetAKeywordField(this.target);
            } else if ( this.target instanceof HTMLTextAreaElement ) {
                 this.onTextInput = true;
            } else if ( this.target instanceof HTMLHtmlElement ) {
               // pages with multiple <body>s are lame. we'll teach them a lesson.
               var bodyElt = this.target.ownerDocument.getElementsByTagName("body")[0];
               if ( bodyElt ) {
                 var computedURL = this.getComputedURL( bodyElt, "background-image" );
                 if ( computedURL ) {
                   this.hasBGImage = true;
                   this.bgImageURL = makeURLAbsolute( bodyElt.baseURI,
                                                      computedURL );
                 }
               }
            } else if ( "HTTPIndex" in content &&
                        content.HTTPIndex instanceof Components.interfaces.nsIHTTPIndex ) {
                this.inDirList = true;
                // Bubble outward till we get to an element with URL attribute
                // (which should be the href).
                var root = this.target;
                while ( root && !this.link ) {
                    if ( root.tagName == "tree" ) {
                        // Hit root of tree; must have clicked in empty space;
                        // thus, no link.
                        break;
                    }
                    if ( root.getAttribute( "URL" ) ) {
                        // Build pseudo link object so link-related functions work.
                        this.onLink = true;
                        this.link = { href : root.getAttribute("URL"),
                                      getAttribute: function (attr) {
                                          if (attr == "title") {
                                              return root.firstChild.firstChild.getAttribute("label");
                                          } else {
                                              return "";
                                          }
                                      }
                                    };
                        // If element is a directory, then you can't save it.
                        if ( root.getAttribute( "container" ) == "true" ) {
                            this.onSaveableLink = false;
                        } else {
                            this.onSaveableLink = true;
                        }
                    } else {
                        root = root.parentNode;
                    }
                }
            }
        }

        // Second, bubble out, looking for items of interest that can have childen.
        // Always pick the innermost link, background image, etc.
        
        const XMLNS = "http://www.w3.org/XML/1998/namespace";
        var elem = this.target;
        while ( elem ) {
            if ( elem.nodeType == Node.ELEMENT_NODE ) {
            
                // Link?
                if ( !this.onLink &&
                     ( (elem instanceof HTMLAnchorElement && elem.href) ||
                        elem instanceof HTMLAreaElement ||
                        elem instanceof HTMLLinkElement ||
                        elem.getAttributeNS( "http://www.w3.org/1999/xlink", "type") == "simple" ) ) {
                    
                    // Target is a link or a descendant of a link.
                    this.onLink = true;
                    this.onMetaDataItem = true;
                    
                    // Remember corresponding element.
                    this.link = elem;
                    this.linkURL = this.getLinkURL();
                    this.linkURI = this.getLinkURI();
                    this.linkProtocol = this.getLinkProtocol();
                    this.onMailtoLink = (this.linkProtocol == "mailto");
                    this.onSaveableLink = this.isLinkSaveable( this.link );
                }

                // Metadata item?
                if ( !this.onMetaDataItem ) {
                    // We display metadata on anything which fits
                    // the below test, as well as for links and images
                    // (which set this.onMetaDataItem to true elsewhere)
                    if ( ( elem instanceof HTMLQuoteElement && elem.cite)    ||
                         ( elem instanceof HTMLTableElement && elem.summary) ||
                         ( elem instanceof HTMLModElement &&
                             ( elem.cite || elem.dateTime ) )                ||
                         ( elem instanceof HTMLElement &&
                             ( elem.title || elem.lang ) )                   ||
                         elem.getAttributeNS(XMLNS, "lang") ) {
                        this.onMetaDataItem = true;
                    }
                }

                // Background image?  Don't bother if we've already found a
                // background image further down the hierarchy.  Otherwise,
                // we look for the computed background-image style.
                if ( !this.hasBGImage ) {
                    var bgImgUrl = this.getComputedURL( elem, "background-image" );
                    if ( bgImgUrl ) {
                        this.hasBGImage = true;
                        this.bgImageURL = makeURLAbsolute( elem.baseURI,
                                                           bgImgUrl );
                    }
                }
            }
            elem = elem.parentNode;
        }
        
        // See if the user clicked on MathML
        const NS_MathML = "http://www.w3.org/1998/Math/MathML";
        if ((this.target.nodeType == Node.TEXT_NODE &&
             this.target.parentNode.namespaceURI == NS_MathML)
             || (this.target.namespaceURI == NS_MathML))
          this.onMathML = true;

        // See if the user clicked in a frame.
        if ( this.target.ownerDocument != window.content.document ) {
            this.inFrame = true;
        }

    },
	
	// Get selected object and convert it to a string to get
    // selected text.   Only display the first 15 chars.
    isTextSelection : function() {
        var result = false;
        var selection = this.searchSelected(16);

        var searchSelectText;
        if (selection) {
            searchSelectText = selection.toString();
            if (searchSelectText.length > 15)
                searchSelectText = searchSelectText.substr(0,15) + "...";
            result = true;
        }
        return result;
    },

    // Returns true if anything is selected.
    isContentSelection : function() {
        return !document.commandDispatcher.focusedWindow.getSelection().isCollapsed;
    },
    
    // Generate fully qualified URL for clicked-on link.
    getLinkURL : function () {
        var href = this.link.href;
        
        if (href) {
          return href;
        }

        var href = this.link.getAttributeNS("http://www.w3.org/1999/xlink",
                                          "href");

        if (!href || !href.match(/\S/)) {
          throw "Empty href"; // Without this we try to save as the current doc, for example, HTML case also throws if empty
        }
        href = makeURLAbsolute(this.link.baseURI, href);
        return href;
    },
    
    getLinkURI : function () {
         var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
         try {
           return ioService.newURI(this.linkURL, null, null);
         } catch (ex) {
           // e.g. empty URL string
           return null;
         }
    },
    
    getLinkProtocol : function () {
        if (this.linkURI) {
            return this.linkURI.scheme; // can be |undefined|
        } else {
            return null;
        }
    },

    // Get text of link.
    linkText : function () {
        var text = gatherTextUnder( this.link );
        if (!text || !text.match(/\S/)) {
          text = this.link.getAttribute("title");
          if (!text || !text.match(/\S/)) {
            text = this.link.getAttribute("alt");
            if (!text || !text.match(/\S/)) {
              text = this.linkURL;
            }
          }
        }

        return text;
    },

	toString : function () {
        return "contextMenu.target     = " + this.target + "\n" +
               "contextMenu.onImage    = " + this.onImage + "\n" +
               "contextMenu.onLink     = " + this.onLink + "\n" +
               "contextMenu.link       = " + this.link + "\n" +
               "contextMenu.inFrame    = " + this.inFrame + "\n" +
               "contextMenu.hasBGImage = " + this.hasBGImage + "\n";
    },
    isTargetATextBox : function ( node )
    {
   
      if (node instanceof HTMLInputElement)
        return (node.type == "text" || node.type == "password")

      return (node instanceof HTMLTextAreaElement);
    },
    isTargetAKeywordField : function ( node )
    {
      var form = node.form;
      if (!form)
        return false;
      var method = form.method.toUpperCase();

      // These are the following types of forms we can create keywords for:
      //
      // method   encoding type       can create keyword
      // GET      *                                 YES
      //          *                                 YES
      // POST                                       YES
      // POST     application/x-www-form-urlencoded YES
      // POST     text/plain                        NO (a little tricky to do)
      // POST     multipart/form-data               NO
      // POST     everything else                   YES
      return (method == "GET" || method == "") ||
             (form.enctype != "text/plain") && (form.enctype != "multipart/form-data");
    },
    
    // Returns a "url"-type computed style attribute value, with the url() stripped.
    getComputedURL: function( elem, prop ) {
         var url = elem.ownerDocument.defaultView.getComputedStyle( elem, '' ).getPropertyCSSValue( prop );
         return ( url.primitiveType == CSSPrimitiveValue.CSS_URI ) ? url.getStringValue() : null;
    },

// Returns true if clicked-on link targets a resource that can be saved.
    isLinkSaveable : function ( link ) {
        // We don't do the Right Thing for news/snews yet, so turn them off
        // until we do.
        return this.linkProtocol && !(
                 this.linkProtocol == "mailto"     ||
                 this.linkProtocol == "javascript" ||
                 this.linkProtocol == "news"       ||
                 this.linkProtocol == "snews"      );
    },
    
    // Temporary workaround for DOM api not yet implemented by XUL nodes.
    cloneNode : function ( item ) {
        // Create another element like the one we're cloning.
        var node = document.createElement( item.tagName );

        // Copy attributes from argument item to the new one.
        var attrs = item.attributes;
        for ( var i = 0; i < attrs.length; i++ ) {
            var attr = attrs.item( i );
            node.setAttribute( attr.nodeName, attr.nodeValue );
        }

        // Voila!
        return node;
    },
    
     searchSelected : function( charlen ) {
        var focusedWindow = document.commandDispatcher.focusedWindow;
        var searchStr = focusedWindow.getSelection();
        searchStr = searchStr.toString();
        // searching for more than 150 chars makes no sense
        if (!charlen)
            charlen = 150;
        if (charlen < searchStr.length) {
            // only use the first charlen important chars. see bug 221361
            var pattern = new RegExp("^(?:\\s*.){0," + charlen + "}");
            pattern.test(searchStr);
            searchStr = RegExp.lastMatch;
        }
        searchStr = searchStr.replace(/^\s+/, "");
        searchStr = searchStr.replace(/\s+$/, "");
        searchStr = searchStr.replace(/\s+/g, " ");
        return searchStr;
    },
    
     // Show/hide one item (specified via name or the item element itself).
    showItem : function ( itemOrId, show ) {
        var item = itemOrId.constructor == String ? document.getElementById(itemOrId) : itemOrId;
        if (item)
          item.hidden = !show;
    },
}
